/*
 * Module Name: blg_interface.c
 *
 * Description: Interface to BLG code from Hager and Zhang.
 *
 * Copyright: Paulo J. S. Silva <pjssilva@gmail.com> 2013.
 */

#ifndef BLG_INTERFACE_H
#define BLG_INTERFACE_H

#include "../../../lib/opt_cont_quad_knapsack.h"

/*
 * Function Name: blg_interface
 *
 * Description: Interface to BLG method from Hager and Zhang. You can
 *     get it at:
 *
 *     http://www.math.ufl.edu/~hager/papers/CG/Archive/
 *
 * Input:
 *     cqk_problem *p: the problem description
 *     double *x0: If non NULL, try to use it as starting point (it is
 *         a primal method).
 *
 * Output:
 *     double *restrict x: solution vector.
 *
 * WARNING: BLGnapsack only work for Euclidean projection, that is
 *    p->d must be composed only of ones. Do not use it otherwise.
 *
 * Return value: the number of iterations required to compute the
 *     solution or -1 in case of failure.
 */
int blg_interface(cqk_problem *restrict in_prob, double *x0, double *x);

#endif
