'''
Prepare adult data to be used vy SVM solver. 

It does two main thigs:

1) Transform the leablled data to numbers (in a very niave way, see
tables below).

2) It applies the normal transformation based on the training data.

Obs: Skip data with missing values.
'''

import scipy as sp

# Conversion tables
workclass = {'Private': 0, 'Self-emp-not-inc': 1, 'Self-emp-inc': 2, 'Federal-gov': 3, 'Local-gov': 4, 'State-gov': 5, 'Without-pay': 6, 'Never-worked': 7}

education = {'Bachelors': 0, 'Some-college': 1, '11th': 2, 'HS-grad': 3, 'Prof-school': 4, 'Assoc-acdm': 5, 'Assoc-voc': 6, '9th': 7, '7th-8th': 8, '12th': 9, 'Masters': 10, '1st-4th': 11, '10th': 12, 'Doctorate': 13, '5th-6th': 14, 'Preschool':15}

marital_status = {'Married-civ-spouse': 0, 'Divorced': 1, 'Never-married': 2, 'Separated': 3, 'Widowed': 4, 'Married-spouse-absent': 5, 'Married-AF-spouse':6}

occupation = {'Tech-support': 0, 'Craft-repair': 1, 'Other-service': 2, 'Sales': 3, 'Exec-managerial': 4, 'Prof-specialty': 5, 'Handlers-cleaners': 6, 'Machine-op-inspct': 7, 'Adm-clerical': 8, 'Farming-fishing': 9, 'Transport-moving': 10, 'Priv-house-serv': 11, 'Protective-serv': 12, 'Armed-Forces': 13}

relationship = {'Wife': 0, 'Own-child': 1, 'Husband': 2, 'Not-in-family': 3, 'Other-relative': 4, 'Unmarried': 5}

race = {'White': 0, 'Asian-Pac-Islander': 1, 'Amer-Indian-Eskimo': 2, 'Other': 3, 'Black': 4}

sex = {'Female': 0, 'Male': 1}

native_country = {'United-States': 0, 'Cambodia': 1, 'England': 2, 'Puerto-Rico': 3, 'Canada': 4, 'Germany': 5, 'Outlying-US(Guam-USVI-etc)': 6, 'India': 7, 'Japan': 8, 'Greece': 9, 'South': 10, 'China': 11, 'Cuba': 12, 'Iran': 13, 'Honduras': 14, 'Philippines': 15, 'Italy': 16, 'Poland': 17, 'Jamaica': 18, 'Vietnam': 19, 'Mexico': 20, 'Portugal': 21, 'Ireland': 22, 'France': 23, 'Dominican-Republic': 24, 'Laos': 25, 'Ecuador': 26, 'Taiwan': 27, 'Haiti': 28, 'Columbia': 29, 'Hungary': 30, 'Guatemala': 31, 'Nicaragua': 32, 'Scotland': 33, 'Thailand': 34, 'Yugoslavia': 35, 'El-Salvador': 36, 'Trinadad&Tobago': 37, 'Peru': 38, 'Hong': 39, 'Holand-Netherlands': 40}

########## Real code starts here

def translate(line):
    '''Convert a line of the data file to numbers using the tables
    above and the class 1 for "<=50K" and -1 for ">50K".
    '''
    cols = line.split(', ')
    cols[1] = workclass[cols[1]]
    cols[3] = education[cols[3]]
    cols[5] = marital_status[cols[5]]
    cols[6] = occupation[cols[6]]
    cols[7] = relationship[cols[7]]
    cols[8] = race[cols[8]]
    cols[9] = sex[cols[9]]
    cols[13] = native_country[cols[13]]
    label = cols.pop()
    if label.startswith('<=50K'):
        cols.insert(0, '1')
    elif label.startswith('>50K'):
        cols.insert(0, '-1')
    else:
        raise ValueError, 'Badly formated line'

    return cols

def compute_normal_scaling(A):
    '''Compute mean and standard deviation of each column of A.
    '''
    return A.mean(axis=0), A.std(axis=0)

def apply_normal_scaling(A, mean, std):
    A -= mean
    A /= std
    return A

def read_file(name):
    '''Read the trainng data set convert it to numbers.
    '''
    translated = []
    for line in file(name):
        if '?' in line: continue
        try:
            translated.append(translate(line))
        except IndexError,ValueError:
            pass
    return sp.array(translated,dtype=sp.float64)

if __name__ == '__main__':
    # Use a fixed random seed to always get the same results
    sp.random.seed(1)

    # Read training data
    print 'Reading training data...'
    train = read_file('../../svm_data/adult/adult.data')
    sp.random.shuffle(train)
    mean, std = compute_normal_scaling(train[:,1:])
    train[:,1:] = apply_normal_scaling(train[:,1:], mean, std)

    # Read test data
    print 'Reading test data...'
    test = read_file('../../svm_data/adult/adult.test')
    test[:,1:] = apply_normal_scaling(test[:,1:], mean, std)

    # Save train and test data
    print 'Saving SVM formatted data...'
    sp.savez('adult.svm', train=train, test=test)

    # Print statistics
    print 'Done.'
    print 'Train positive:', (train[:,0] == 1).sum()
    print 'Train negative:', (train[:,0] == -1).sum()
    print 'Train total:   ', train.shape[0]
    print 'Test postive:  ',  (test[:,0] == 1).sum()
    print 'Test negative: ',  (test[:,0] == -1).sum()
    print 'Test total:    ',  test.shape[0]
