'''
Download the UCI adult database if necessary.

The database is located at:

http://archive.ics.uci.edu/ml/machine-learning-databases/adult/
'''

import os
import os.path as path
import sys
import urllib
import md5

# Used to reports the amount downloade, similar to wget.
#
# Tip from
# http://stackoverflow.com/questions/~/wget-vs-urlretrieve-of-python
def reporthook(a,b,c): 
    # ',' at the end of the line is important!
    print "% 3.1f%% of %d bytes\r" % (min(100, float(a * b) / c * 100), c),
    #you can also use sys.stdout.write
    #sys.stdout.write("\r% 3.1f%% of %d bytes" 
    #                 % (min(100, float(a * b) / c * 100), c)

# Used to compute the md5 digest of a file in a memory efficient way.
#
# Tip from
# http://stackoverflow.com/questions/1131220/get-md5-hash-of-a-files-without-open-it-in-python
def md5sum(filename):
    digest = md5.new()
    with open(filename,'rb') as f: 
        for chunk in iter(lambda: f.read(128*digest.block_size), b''): 
             digest.update(chunk)
    return digest.hexdigest()

def accessible(url):
    '''Verify if an url is acessible.
    '''
    filehandle = urllib.urlopen(url)
    return filehandle.code == 200
    
def download(filename, filemd5, base_url):
    '''Download the file name from base_url and verify its md5 digest.
    '''
    if not path.exists(filename):
        url = base_url + filename
        if not accessible(url):
            print >>sys.stderr, url,
            print >>sys.stderr, 'not accessible.'
            print >>sys.stderr, 'Please, try again later or contact pjssilva@ime.usp.br'
            sys.exit(1)
        print 'Downloading', filename
        urllib.urlretrieve(url, filename, reporthook)
        print '\nChecking md5...',
        if md5sum(filename) != filemd5:
            print >>sys.stderr, url,
            print >>sys.stderr, 'MD5 digest does not match.'
            print >>sys.stderr, 'The file has changed in the repository.'
            print >>sys.stderr, 'It is not possible to guarantee that the'
            print >>sys.stderr, 'results will remain the same. Please, contact'
            print >>sys.stderr, 'pjssilva@ime.usp.br if necessary.'
        else:
            print 'OK'

if __name__ == '__main__':
    base_url = 'http://archive.ics.uci.edu/ml/machine-learning-databases/adult/'
    files_md5 = (('adult.data', '5d7c39d7b8804f071cdd1f2a7c460872'),
             ('adult.names', '1a7cdb3ff7a1b709968b1c7a11def63e'),
             ('adult.test', '35238206dfdf7f1fe215bbb874adecdc'))

    for name, digest in files_md5:
        download(name, digest, base_url)
    

